/*
 * A process server providing a way of creating new processes.
 *
 * Copyright (C) 2023 Paul Boddie <paul@boddie.org.uk>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02110-1301, USA
 */

#include <l4/sys/debugger.h>
#include <l4/sys/err.h>

#include <fsclient/client.h>
#include <ipc/thread.h>

#include <stdio.h>
#include <stdlib.h>

#include <exec/process_creator_resource.h>
#include <resource/resource_server.h>
#include <systypes/env.h>
#include <systypes/fcntl.h>



/* Server program. */

int main(int argc, char *argv[])
{
  l4_debugger_set_object_name(l4re_env()->main_thread, "process_server");

  if (argc < 2)
  {
    printf("Need a region mapper.\n");
    return 1;
  }

  char *rm_filename = argv[1];
  const char *server_name = (argc > 2) ? argv[2] : ENV_PROCESS_SERVER_NAME;
  long err;

  /* Introduce concurrency control. */

  err = ipc_thread_init();

  if (err)
  {
    printf("Initialisation error: %s\n", l4sys_errtostr(err));
    return 1;
  }

  file_t *rm_file = client_open(rm_filename, O_RDONLY);

  if (!client_opened(rm_file))
  {
    err = (rm_file == NULL) ? -L4_ENOMEM : rm_file->error;
    printf("Could not open region mapper file: %s\n", l4sys_errtostr(err));
    return 1;
  }

  ProcessCreatorResource creator(rm_filename, rm_file);

  /* Register a server associating it with the given object. */

  ResourceServer server(&creator);
  err = server.bind(server_name);

  if (err)
  {
    printf("Could not bind server: %s\n", l4sys_errtostr(err));
    return 1;
  }

  printf("Starting server...\n");
  server.start();
  return 0;
}

/* vim: tabstop=2 expandtab shiftwidth=2
*/
